/*
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "toptransparentwidget.h"
#include "notifyreceiveinfo.h"
#include <QPainterPath>
#include <QDBusMessage>
#include <QDBusConnection>
#include <QTime>

topTransparentWidget::topTransparentWidget(QWidget *parent) : QWidget(parent)
{
    m_pMainLayout = new QVBoxLayout();
    m_pMainLayout->setContentsMargins(0, 0, 0, 0);
    m_pMainLayout->setDirection(QBoxLayout::BottomToTop);
    m_pMainLayout->setSpacing(5);
    this->setContentsMargins(0, 0, 0, 0)    ;

    m_pSreenInfo = new adaptScreenInfo();

    connect(this, &topTransparentWidget::closePopupWidget, this, &topTransparentWidget::moveAllpopWidgetSiteAccordId);

    initPanelSite();
    setNotifyPopWidgetSite();
    if(PLATFORM::g_platformType == PLATFORM::V101){
        connect(QApplication::desktop(), &QDesktopWidget::resized, this, &topTransparentWidget::updataPopWidgetSiteSlots);
        connect(m_pSreenInfo, &adaptScreenInfo::updataTopWidgetSite, this, [=](){
            setNotifyPopWidgetSite();
        });
    }
    if(PLATFORM::g_platformType == PLATFORM::Intel){
        //connect参数：发送者、路径、interface、信号名、接受者、槽函数
        QDBusConnection::sessionBus().connect(QString(""), QString("/org/ukui/Sidebar"),
                                              "org.ukui.Sidebar", "animationAction",
                                              this, SLOT(animationActionSlots(uint,int)));
        m_posAnimation = new QPropertyAnimation(this, "pos");

        connect(m_pSreenInfo, &adaptScreenInfo::updataTopWidgetSite, this, [=](){
            setNotifyPopWidgetSite();
        });
    }



    this->setFixedWidth(372);
    this->setLayout(m_pMainLayout);
    setWidgetFlag();
    return;
}

void topTransparentWidget::setWidgetFlag()
{
    setWindowFlags(Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint);
    setAttribute(Qt::WA_TranslucentBackground);
    return;
}

void topTransparentWidget::setWidgetSize(int width, int heigh)
{
    this->setFixedSize(width, heigh);
    return;
}

void topTransparentWidget::setWidgetPos(int x, int y)
{
    qInfo() << "set pop-up window X:" << x;
    qInfo() << "set pop-up window Y:" << y;
    this->move(x, y);
    return;
}

/* 添加弹窗 */
void topTransparentWidget::AddPopupItemWidget(notifyReceiveInfo *entryInfo)
{
    if (entryInfo == nullptr) {
        return;
    }
    popupItemWidget *popw = new popupItemWidget(this,entryInfo);

    if (popWidgetqueue.count() == 0) {
        m_ListWidgetHeight = 0;
    }

    //判断是否设置定时，若设置定时，走独立定时分支。若不设置定时，累加定时器
    if (entryInfo->timeout().toInt() == -1) {  //默认驻留时间
        if(popWidgetqueue.count() >= m_fixNotifyNum){
           popw->m_poutTimer->setInterval(3000 * (popWidgetqueue.count() - m_fixNotifyNum + 1));
        }
        else {
            popw->m_poutTimer->setInterval(3000 * (popWidgetqueue.count() + 1));
        }
    }
    else if(entryInfo->timeout().toInt() == 0){  //弹窗常驻
        ++m_fixNotifyNum;
        popw->m_poutTimer->blockSignals(true);
        popw->m_quitTimer->blockSignals(true);
    }
    else {                                      //应用自定义驻留时间
        popw->m_poutTimer->setInterval(entryInfo->timeout().toInt()*1);
    }

    connect(popw, &popupItemWidget::timeout, this, &topTransparentWidget::moveAllpopWidgetSite);
    connect(popw, &popupItemWidget::mouseMissed, this, &topTransparentWidget::mouseMissedSlots);
    connect(popw, &popupItemWidget::timeOutMissed, this, &topTransparentWidget::timeOutMissedSlots);
    connect(popw, &popupItemWidget::clickedMissed, this, &topTransparentWidget::clickedMissedSlots);
    connect(popw, &popupItemWidget::actionInvokedMissed, this, &topTransparentWidget::actionInvokedMissedSlots);
    connect(popw, &popupItemWidget::actionButtonClicked, this, &topTransparentWidget::actionInvokedMissedSlots);
    connect(popw, &popupItemWidget::animationAction, this, &topTransparentWidget::TransformGroundGlassAreaSlots);
    connect(popw->m_pOutAnimation, &QPropertyAnimation::finished, \
            this, &topTransparentWidget::addWaittingPopupWidgetSlots);
    popw->setEntryData(entryInfo);
    popWidgetqueue.append(popw);

    if (popWidgetqueue.count() == 1) {
        m_ListWidgetHeight += popw->height();
        this->setFixedHeight(m_ListWidgetHeight);
        m_pMainLayout->addWidget(popw);
        popw->setGeometry(0, 0, popw->width(), popw->height());
        this->update();
   } else {
        if(PLATFORM::g_platformType == PLATFORM::V101){
            if (m_ListWidgetHeight < m_pSreenInfo->m_screenHeight - 134) {
                m_ListWidgetHeight += popw->height() + 5;
                this->setFixedHeight(m_ListWidgetHeight);
                m_pMainLayout->addWidget(popw);
                popw->setGeometry(0, 0, popw->width(), popw->height());
                this->update();
            } else {
                m_pWaitingQueue.append(entryInfo);
                popWidgetqueue.removeAt(popWidgetqueue.count() - 1);
                delete popw;
            }
        }
        else if(PLATFORM::g_platformType == PLATFORM::Intel){
            if (m_ListWidgetHeight < m_pSreenInfo->m_screenHeight - 250) {
               m_ListWidgetHeight += popw->height() + 5;
               this->setFixedHeight(m_ListWidgetHeight);
               m_pMainLayout->addWidget(popw);
               popw->setGeometry(0, 0, popw->width(), popw->height());
               this->update();
           } else {
               m_pWaitingQueue.append(entryInfo);
               popWidgetqueue.removeAt(popWidgetqueue.count() - 1);
               delete popw;
           }
        }
    }
    return;
}

/* 替换通知弹窗的内容 */
void topTransparentWidget::replacePopupItemWidget(uint oldInfoId, notifyReceiveInfo *replaceInfo)
{
    for(int i=0; i<popWidgetqueue.size(); ++i){
        popupItemWidget* popwidget = popWidgetqueue.at(i);
        if(popwidget->m_pentryInfo->id().toUInt() == oldInfoId){
            popwidget->updateWidgetDate(replaceInfo);
        }
    }
}

void topTransparentWidget::setWidgetLayout(QWidget *widget)
{
    m_pMainLayout->addWidget(widget);
    this->setLayout(m_pMainLayout);
    return;
}

/* 隐藏超时的widget弹窗界面 */
void topTransparentWidget::exitPopupWidget(QWidget *w)
{
    popupItemWidget* popw = dynamic_cast<popupItemWidget *>(w);
    m_ListWidgetHeight = m_ListWidgetHeight - w->height() - 5;
    m_pMainLayout->removeWidget(popw);
    popWidgetqueue.removeOne(popw);
    delete popw->m_pentryInfo;
    this->setFixedHeight(m_ListWidgetHeight);
    if (0 == popWidgetqueue.count()) {
        this->hide();
        return;
    }
    this->update();
    return;
}

void topTransparentWidget::addEntryInfo(notifyReceiveInfo *entryInfo)
{
    m_entities.enqueue(entryInfo);
    return;
}

void topTransparentWidget::consumeEntities()
{
    if (!m_currentNotify.isNull()) {
        m_currentNotify->deleteLater();
        m_currentNotify = nullptr;
    }

    if (m_entities.isEmpty()) {
        m_currentNotify = nullptr;
        qApp->quit();
        return;
    }

    m_currentNotify = m_entities.dequeue();
    this->setProperty("blurRegion", QRegion(QRect(0, 0, 372, this->height())));
    qDebug()<<">>>>>>>>>>>>>1";
    AddPopupItemWidget(m_currentNotify);
    if (!this->isVisible())
        this->show();
    return;
}

// 初始化任务栏gsetting
void topTransparentWidget::initPanelSite()
{
    /* 链接任务栏dgsetting接口 */
    if(QGSettings::isSchemaInstalled(UKUI_PANEL_SETTING))
        m_pPanelSetting = new QGSettings(UKUI_PANEL_SETTING);

    if (m_pPanelSetting != nullptr) {
        connect(m_pPanelSetting, &QGSettings::changed, this, &topTransparentWidget::panelSiteSlots);
        QStringList keys = m_pPanelSetting->keys();

        /* 获取任务栏位置 */
        if (keys.contains("panelposition")) {
            m_ipanelPosition = m_pPanelSetting->get("panelposition").toInt();
        }

        /* 获取任务栏高度 */
        if (keys.contains("panelsize")) {
            m_ipanelHeight = m_pPanelSetting->get("panelsize").toInt();
        }
    }
    return;
}

/* 设置通知弹窗的位置 */
void topTransparentWidget::setNotifyPopWidgetSite()
{
    qDebug()<<">>>>>设置通知弹窗的位置>";
    if(PLATFORM::g_platformType == PLATFORM::V101){
        QDBusInterface iface("org.ukui.panel",
                             "/panel/position",
                             "org.ukui.panel",
                             QDBusConnection::sessionBus());
        if (iface.isValid()) {
            QDBusReply < QVariantList > reply = iface.call("GetPrimaryScreenGeometry");
            QVariantList position_list = reply.value();
//            m_iScreenXGeometry = position_list.at(0).toInt();
//            m_iScreenYGeometry = position_list.at(1).toInt();
//            //m_iScreenWidth     = position_list.at(2).toInt();
//            //m_iScreenHeight    = position_list.at(3).toInt();
            m_ipanelPosition   = position_list.at(4).toInt();

//            m_iScreenWidth = QGuiApplication::primaryScreen()->geometry().width();
//            m_iScreenHeight = QGuiApplication::primaryScreen()->geometry().height();
        }
//        else {
//            m_iScreenXGeometry = m_pSreenInfo->m_nScreen_x;
//            m_iScreenYGeometry = m_pSreenInfo->m_nScreen_y;
//            m_iScreenWidth     = m_pSreenInfo->m_screenWidth;
//            m_iScreenHeight    = m_pSreenInfo->m_screenHeight;
//            m_ipanelPosition   = 0;
//        }

//        switch (m_ipanelPosition) {
//            case 1:
//                move(m_iScreenXGeometry + m_iScreenWidth - 372 - MARGIN,
//                     m_iScreenYGeometry + MARGIN);
//                break;
//            case 2:
//                move(m_iScreenXGeometry + m_iScreenWidth - 372 - m_ipanelHeight - MARGIN,
//                     m_iScreenYGeometry + MARGIN);
//                break;
//            case 3:
//                move(m_iScreenXGeometry + m_iScreenWidth - 372 - m_ipanelHeight - MARGIN,
//                     m_iScreenYGeometry + MARGIN);
//                break;
//            default:
//                move(m_iScreenXGeometry + m_iScreenWidth - 372 - MARGIN,
//                     m_iScreenYGeometry + MARGIN);
//                break;
//        }

        /////////////////////////
        //获取当前主屏幕信息
        int primaryScreenX = QApplication::primaryScreen()->geometry().x();
        int primaryScreenY = QApplication::primaryScreen()->geometry().y();
        int primaryScreenWidth = QApplication::primaryScreen()->geometry().width();
        qDebug()<<">>>>>>"<<primaryScreenX<<primaryScreenY<<primaryScreenWidth;

        //根据任务栏的位置和侧边栏位置，决定通知弹窗的显示位置
        switch (m_ipanelPosition) {
            case topTransparentWidget::PanelDown:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - 8), primaryScreenY + 5);
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - 8));
                m_point.setY(primaryScreenY + 5);
                break;
            case topTransparentWidget::PanelUp:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - 8), primaryScreenY + 5 + m_ipanelHeight); // 获取任务栏高度
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - 8));
                m_point.setY(primaryScreenY + 5 + m_ipanelHeight);
                break;
            case topTransparentWidget::PanelLeft:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - 8), primaryScreenY + 5);
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - 8));
                m_point.setY(primaryScreenY + 5);
                break;
            case topTransparentWidget::PanelRight:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - 8 - m_ipanelHeight), primaryScreenY + 5);
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - 8 - m_ipanelHeight));
                m_point.setY(primaryScreenY + 5);
                break;
            default:
                break;
        }
    }
    else if(PLATFORM::g_platformType == PLATFORM::Intel){
        //--> 根据侧边栏位置显示
        //QTime _Timer = QTime::currentTime().addMSecs(5000);
        //while( QTime::currentTime() < _Timer );
        int sidebarWidth = 0;
        //获取侧边栏显示的宽度， createMethodCall参数：目的地，路径，interfaceming,方法名
        QDBusMessage msg = QDBusMessage::createMethodCall("org.ukui.Sidebar", "/org/ukui/Sidebar",
                                                        "org.ukui.Sidebar", "getWidth");
        QDBusMessage response = QDBusConnection::sessionBus().call(msg);
        if (response.type() == QDBusMessage::ReplyMessage){
            sidebarWidth = response.arguments().takeFirst().toInt();
        }
        else{
            qCritical() << "getWidth error:" << response.type();
            sidebarWidth = 0;
        }
        //获取当前屏幕旋转状态  (四种状态 "normal" "upside-down" "left" "right")
        QString rotationState = "normal";
        QDBusMessage rotationMsg = QDBusMessage::createMethodCall("com.kylin.statusmanager.interface", "/",
                                                        "com.kylin.statusmanager.interface", "get_current_rotation");
        QDBusMessage response2 = QDBusConnection::sessionBus().call(rotationMsg);
        if (response2.type() == QDBusMessage::ReplyMessage){
            rotationState = response2.arguments().takeFirst().toString();
        }
        else{
            qCritical() << "get_current_rotation error:" << response.type();
        }

        //获取当前主屏幕信息
        int primaryScreenX = QApplication::primaryScreen()->geometry().x();
        int primaryScreenY = QApplication::primaryScreen()->geometry().y();
        int primaryScreenWidth = QApplication::primaryScreen()->geometry().width();

        //根据任务栏的位置和侧边栏位置，决定通知弹窗的显示位置
        switch (m_ipanelPosition) {
            case topTransparentWidget::PanelDown:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - sidebarWidth - 8), primaryScreenY + 5);
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - sidebarWidth - 8));
                m_point.setY(primaryScreenY + 5);
                break;
            case topTransparentWidget::PanelUp:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - sidebarWidth - 8), primaryScreenY + 5 + m_ipanelHeight); // 获取任务栏高度
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - sidebarWidth - 8));
                m_point.setY(primaryScreenY + 5 + m_ipanelHeight);
                break;
            case topTransparentWidget::PanelLeft:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - 8), primaryScreenY + 5);
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - 8));
                m_point.setY(primaryScreenY + 5);
                break;
            case topTransparentWidget::PanelRight:
                setWidgetPos(primaryScreenX + (primaryScreenWidth - 372 - sidebarWidth - 8 - m_ipanelHeight), primaryScreenY + 5);
                m_point.setX(primaryScreenX + (primaryScreenWidth - 372 - sidebarWidth - 8 - m_ipanelHeight));
                m_point.setY(primaryScreenY + 5);
                break;
            default:
                break;
        }
    }
    return;
}

bool topTransparentWidget::isPopupExist(uint replacesId)
{
    for(int i=0; i<popWidgetqueue.size(); ++i){
        popupItemWidget* popwidget = popWidgetqueue.at(i);
        if(popwidget->m_pentryInfo->id().toUInt() == replacesId)
            return true;
    }
    return false;
}

// hash表插入
void topTransparentWidget::deleteHashInsert(popupItemWidget* key, QListWidgetItem* value)
{
    if (nullptr == key || nullptr == value) {
        qDebug() << "插入函数形参为Null";
        return;
    }
    if (deleteHash.value(key)) {
        qDebug() << "注册：value已存在";
        return;
    }
    deleteHash.insert(key, value);
    return;
}

// hash表移除键值对
void topTransparentWidget::deleteHashRemove(popupItemWidget* key)
{
    if (key == nullptr) {
        qWarning() << "移除键值函数形参--->key为空";
        return;
    }
    if (deleteHash.contains(key))
        deleteHash.remove(key);
    return;
}

// hash表查找
QListWidgetItem* topTransparentWidget::deleteHashSearch(popupItemWidget* key)
{
    if (key == nullptr) {
        qWarning() << "查找函数形参-----> *key为空";
        return nullptr;
    }

    if (deleteHash.contains(key))
        return deleteHash.value(key);
    else
        return nullptr;
}

//重新绘制背景色
void topTransparentWidget::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    QPainterPath path;
    opt.rect.adjust(0,0,0,0);
    path.addRoundedRect(opt.rect,6,6);
    p.setRenderHint(QPainter::Antialiasing);                        //反锯齿
//    setProperty("blurRegion", QRegion(path.toFillPolygon().toPolygon()));
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
    QWidget::paintEvent(event);
}

void topTransparentWidget::mouseMissedSlots(QWidget *w, int id)
{
    --m_fixNotifyNum;
    exitPopupWidget(w);
    emit dismissed(id);
    return;
}

void topTransparentWidget::timeOutMissedSlots(QWidget *w, int id)
{
    exitPopupWidget(w);
    emit dismissed(id);
    return;
}

void topTransparentWidget::clickedMissedSlots(QWidget *w, int id)
{
    --m_fixNotifyNum;
    exitPopupWidget(w);
    emit dismissed(id);
    return;
}

void topTransparentWidget::actionInvokedMissedSlots(QWidget *w, int id, QString actionId)
{
    --m_fixNotifyNum;
    w->hide();
    exitPopupWidget(w);
    emit actionInvoked(id, actionId);
    return;
}

void topTransparentWidget::actionButtonClickedSlots(QWidget *w, int id, QString actionId)
{
    w->hide();
    exitPopupWidget(w);
    emit actionInvoked(id, actionId);
    return;
}

// 设置所有widget坐标，调用隐藏动画动画
void topTransparentWidget::moveAllpopWidgetSite(QWidget* w)
{
    popupItemWidget* popw = dynamic_cast<popupItemWidget *>(w);
    int siteHeight = m_ListWidgetHeight;
    for (int i = 0; i < popWidgetqueue.count(); i++) {
        popWidgetqueue.at(i)->setGeometry(0, siteHeight - popWidgetqueue.at(i)->height(), \
                                          popWidgetqueue.at(i)->width(), popWidgetqueue.at(i)->height());

        siteHeight = siteHeight - popWidgetqueue.at(i)->height() - 5;
        if (popw == popWidgetqueue.at(i)) {
            popw->stowToSidebar();
            popWidgetqueue.at(i)->m_pOutAnimation->setStartValue(popWidgetqueue[i]->geometry());
            popWidgetqueue.at(i)->m_pOutAnimation->setEndValue(QRect(this->width() + 10, popWidgetqueue[i]->geometry().y(), this->width(), 88));
            popWidgetqueue.at(i)->m_pOutAnimation->start();
        }
    }
    return;
}

/* 通过id关闭对应的弹窗窗口 */
void topTransparentWidget::moveAllpopWidgetSiteAccordId(int Id)
{
    int siteHeight = m_ListWidgetHeight;
    for (int i = 0; i < popWidgetqueue.count(); i++) {
        popupItemWidget* popupWidget = popWidgetqueue.at(i);
        if(popupWidget->m_pentryInfo->id().toUInt() == Id){
            popupWidget->stowToSidebar();
            popupWidget->hide();
            exitPopupWidget(popupWidget);
            popWidgetqueue.removeAt(i);
            i--;
            continue;
        }


//        popWidgetqueue.at(i)->setGeometry(0, siteHeight - popWidgetqueue.at(i)->height(),
//                                            popWidgetqueue.at(i)->width(), popWidgetqueue.at(i)->height());
//        siteHeight = siteHeight - popWidgetqueue.at(i)->height() - 5;
//        if (Id == popWidgetqueue.at(i)->m_pentryInfo->replacesId().toInt()) {
//            popWidgetqueue.at(i)->m_pOutAnimation->setStartValue(popWidgetqueue[i]->geometry());
//            popWidgetqueue.at(i)->m_pOutAnimation->setEndValue(QRect(this->width() + 10, popWidgetqueue[i]->geometry().y(), this->width(), 88));
//            popWidgetqueue.at(i)->m_pOutAnimation->start();
//        }
    }
    return;
}

void topTransparentWidget::TransformGroundGlassAreaSlots(const QVariant &value, QWidget *w)
{
    if (popWidgetqueue.count() == 1) {
        this->setProperty("blurRegion", QRegion(QRect(0, 0, 1, this->height() - w->height() + 1)));
    } else {
        this->setProperty("blurRegion", QRegion(QRect(0, 0, 372, this->height() - w->height())));
    }
}

void topTransparentWidget::addWaittingPopupWidgetSlots()
{
    if (m_pWaitingQueue.count() != 0) {
        AddPopupItemWidget(m_pWaitingQueue.at(0));
        m_pWaitingQueue.removeAt(0);
    }
    return;
}

void topTransparentWidget::panelSiteSlots(QString key)
{
    if (key == "panelposition" || key == "panelsize") {
        m_ipanelPosition = m_pPanelSetting->get("panelposition").toInt();
        m_ipanelHeight   = m_pPanelSetting->get("panelsize").toInt();
        setNotifyPopWidgetSite();
    }
    return;
}

void topTransparentWidget::updataPopWidgetSiteSlots(int screen)
{
    setNotifyPopWidgetSite();
}

/*参数：
 * time:动画持续时间
 * distance：移动距离
*/
void topTransparentWidget::animationActionSlots(const uint time, const int distance)
{
    /* 获取任务栏位置 */
    int m_ipanelPosition = 0;
    if (m_pPanelSetting->keys().contains("panelposition")) {
        m_ipanelPosition = m_pPanelSetting->get("panelposition").toInt();
    }else{
        return;
    }
    if(m_ipanelPosition != topTransparentWidget::PanelLeft){
        QPoint startPoint;
        QPoint endPoint;
        startPoint = m_point;
        endPoint   = m_point - QPoint(distance,0);
        m_point = endPoint;//记录当前结束位置，作为下一次的开始位置
        m_posAnimation->stop();
        m_posAnimation->setDuration(time);
        m_posAnimation->setStartValue(startPoint);
        m_posAnimation->setEndValue(endPoint);
        m_posAnimation->start();
    }
}
